import os
import typer
import pathlib
from autoserve import __version__


app = typer.Typer(add_completion=False)
curr_dir = pathlib.Path(os.getcwd())


@app.command()
def spelling(
    port: int = typer.Option(8501, help="Port number"),
    model_folder: pathlib.Path = typer.Option(
        curr_dir / "models", help="Folder that contains all Rasa NLU models"
    ),
    project_folder: pathlib.Path = typer.Option(
        curr_dir, help="The Rasa project folder (for custom component paths"
    ),
):
    """Check the effect of spelling on NLU predictions."""
    pass
    
    
@app.command()
def version():
    """Prints the current version"""
    print(f"{__version__}")

app()


import setuptools
from glob import glob

__package_name__ = "autoserve"
__version__ = "1.0"
__repository_url__ = ""
__download_url__ = ""

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
with open('requirements.txt') as f:
    required = f.read().splitlines()

setuptools.setup(
    name=__package_name__,
    version=__version__,
    license='MIT',
    author="subhrajit",
    author_email="techbiglover@gmail.com",
    description="Model Serving Automation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=__repository_url__,
    download_url = __download_url__,
    packages=setuptools.find_packages(
        exclude=(
            "test.py"
            )
        ),
    include_package_data=True,
    data_files=[
        ('DB', glob('autoserve/meta/*.db')),
    ],
    entry_points={
        "console_scripts": [
            "autoserve=autoserve.__main__:app",
        ],
    },
    python_requires='>=3.7, <4',
    install_requires = required
)


from kubernetes import client, config
from subprocess import run, Popen, PIPE

config.load_kube_config()
k8s_beta = client.ExtensionsV1beta1Api()

def secret(DEPLOY_NAMESPACE, MINIO_ACCESS_KEY, MINIO_SECRET_KEY, MINIO_HOST, SECRET_NAME):
    secret = f"""apiVersion: v1
kind: Secret
metadata:
  name: {SECRET_NAME}
  namespace: {DEPLOY_NAMESPACE}
type: Opaque
stringData:
  AWS_ACCESS_KEY_ID: {MINIO_ACCESS_KEY}
  AWS_SECRET_ACCESS_KEY: {MINIO_SECRET_KEY}
  AWS_ENDPOINT_URL: http://{MINIO_HOST}
  USE_SSL: "false"
"""

    resp = k8s_beta.create_namespaced_deployment(
                body=secret, namespace=DEPLOY_NAMESPACE)
    print("Deployment created. status='%s'" % str(resp.status))

    ret = False
    
    if str(resp.status) == "Created":
          ret = True
          
    return ret



def serviceAccount(DEPLOY_NAMESPACE, SECRET_NAME):
    sa = f"""apiVersion: v1
kind: ServiceAccount
metadata:
  name: minio-sa
  namespace: {DEPLOY_NAMESPACE}
secrets:
  - name: {SECRET_NAME}
"""

    resp = k8s_beta.create_namespaced_deployment(
                body=sa, namespace=DEPLOY_NAMESPACE)
    print("Deployment created. status='%s'" % str(resp.status))

    ret = False
    
    if str(resp.status) == "Created":
          ret = True
          
    return ret

def seldonDeployement(MODEL_NAME, DEPLOY_NAMESPACE, MINIO_MODEL_BUCKET, INCOME_MODEL_PATH, SECRET_NAME):
    seldon_deployement = f"""apiVersion: machinelearning.seldon.io/v1
kind: SeldonDeployment
metadata:
  name: {MODEL_NAME}
  namespace: {DEPLOY_NAMESPACE}
spec:
  predictors:
  -  graph:
      implementation: SKLEARN_SERVER
      modelUri: s3://{MINIO_MODEL_BUCKET}/{INCOME_MODEL_PATH}
      envSecretRefName: {SECRET_NAME}
      name: classifier
      logger:
         mode: all
     name: default
     replicas: 1
"""

    resp = k8s_beta.create_namespaced_deployment(
                body=seldon_deployement, namespace=DEPLOY_NAMESPACE)
    print("Deployment created. status='%s'" % str(resp.status))

    ret = False
    if str(resp.status) == "Created":
          ret = True
          
    return ret


def main(MODEL_NAME, 
         DEPLOY_NAMESPACE, 
         MINIO_ACCESS_KEY, 
         MINIO_SECRET_KEY, 
         MINIO_HOST, 
         MINIO_MODEL_BUCKET, 
         INCOME_MODEL_PATH
):
    
    sucessful_deployement = False
    
    SECRET_NAME = "seldon-init-container-secret"
        
    secret = secret(DEPLOY_NAMESPACE, MINIO_ACCESS_KEY, MINIO_SECRET_KEY, MINIO_HOST, SECRET_NAME)
    service_account = serviceAccount(DEPLOY_NAMESPACE, SECRET_NAME)
    seldon_deployement = seldonDeployement(MODEL_NAME, DEPLOY_NAMESPACE, MINIO_MODEL_BUCKET, INCOME_MODEL_PATH, SECRET_NAME)
    
    if secret and service_account and seldon_deployement:
          sucessful_deployement = True
          
    return sucessful_deployement
    

def testModel_clusterIP(MODEL_NAME, DATA, DEPLOY_NAMESPACE):
      cmd=f"""curl -d '{DATA}' \
        http://{MODEL_NAME}-default.{DEPLOY_NAMESPACE}:8000/api/v1.0/predictions \
        -H "Content-Type: application/json"
      """
    
      ret = Popen(cmd, shell=True,stdout=PIPE)
      return ret.stdout.read().decode("utf-8")
    
def testModel_LoadBalancer(LOAD_BALANCER, MODEL_NAME, DATA, DEPLOY_NAMESPACE):
      cmd=f"""curl -d '{DATA}' \
        http://{LOAD_BALANCER}/seldon/{DEPLOY_NAMESPACE}/{MODEL_NAME}/api/v0.1/predictions \
        -H "Content-Type: application/json"
      """
    
      ret = Popen(cmd, shell=True,stdout=PIPE)
      return ret.stdout.read().decode("utf-8")